local QuickAuctions = LibStub("AceAddon-3.0"):GetAddon("QuickAuctions", true)
	or LibStub("AceAddon-3.0"):GetAddon("ZeroAuctions", true)
	or LibStub("AceAddon-3.0"):GetAddon("AuctionProfitMaster", true)

if not QuickAuctions then
	return
end

local QAManager = select(2, ...)
QAManager = LibStub("AceAddon-3.0"):NewAddon(QAManager, "QAManager", "AceEvent-3.0")
--[===[@debug@
	_G['QAManager'] = QAManager
--@end-debug@]===]



local profile, parseMoney, formatMoney


local function simpleGet(info)
	local key = info[#(info)]
	return profile[key]
end

local function simpleSet(info, value)
	local key = info[#(info)]
	profile[key] = value
end

local function FallbackBases()
	options = {
		['Auctioneer:MP'] = 'Auctioneer Market Price',
		['Auctioneer:Low'] = 'Auctioneer Lowest Price',
	}
	if GetAuctionBuyout ~= nil then
		options['GetAuctionBuyout'] = 'Other (GetAuctionBuyout compatibile)'
	end

	return options
end

local Options = {
	handler = QAManager,
	name = "QAManager r13",
	type = 'group',
	childGroups = 'tab',
	args = {
		config = {
			type = "execute",
			name = "config",
			desc = "Show Blizzard's options GUI",
			func = "ShowOptionsGUI",
			guiHidden = true,
		},
		refresh = {
			type = "execute",
			name = "Refresh QA Thresholds",
			desc = "Resets all Quick Auctions thresholds",
			func = "RefreshQAGroups",
			order = 15,
		},
		threshold = {
			name = "Threshold",
			type = 'group',
			args = {
				description = {
					type = "description",
					name = function()
						return format("Thresholds will be kept at ItemAuditor Cost + %s%% + %s%% (AH Fees) or ItemAuditor Cost + %s + %s%% (AH Fees), which ever is higher for the highest item in the group.",
						floor(profile.threshold_percent*100),
						floor(profile.auction_house*100),
						formatMoney(profile.threshold_value),
						floor(profile.auction_house*100))
					end,
					order = 0,
				},
				manage_threshold = {
					type = "toggle",
					name = "Manage Thresholds",
					get = simpleGet,
					set = simpleSet,
					order = 1,
				},
				threshold_base = {
					type = "select",
					name = "Threshold Base",
					disabled = function() return not profile.manage_threshold end,
					desc = "",
					values = {
						ItemAuditor = 'ItemAuditor Cost',
					},
					get = simpleGet,
					set = simpleSet,
					order = 2,
				},

				threshold_percent = {
					type = "range",
					name = "% Threshold",
					desc = "Don't sell items for less than this amount of profit.",
					disabled = function() return not profile.manage_threshold end,
					min = 0.0,
					max = 1.0,
					step = 0.01,
					isPercent = true,
					get = simpleGet,
					set = simpleSet,
					order = 6,
				},

				threshold_value = {
					type = "input",
					name = "Value Threshold",
					desc = "Don't sell items for less than this amount of profit.",
					disabled = function() return not profile.manage_threshold end,
					get = function()
						return formatMoney(profile.threshold_value , '', true)
					end,
					validate = function(info, value)
						if parseMoney(value) == nil then
							return "Invalid money format"
						end
						return true
					end,
					set = function(info, value)
						profile.threshold_value = parseMoney(value)
					end,
					usage = "###g ##s ##c",
					order = 8,
				},
				auction_house = {
					type = "select",
					name = "Auction House",
					desc = "",
					values = {
						[0.05] = UnitFactionGroup("player"),
						[0.15] = 'Neutral'
					},
					get = simpleGet,
					set = simpleSet,
				},
			}
		},
		fallback = {
			name = "Fallback",
			type = 'group',
			args = {
				description = {
					type = "description",
					name = function()
						local bases = FallbackBases()
						return format("Fallbacks will be set to %s + %s%% for the most expensive item in the group.",
						bases[profile.fallback_base],
						floor(profile.fallback_percent*100))
					end,
					order = 0,
				},
				manage_fallback = {
					type = "toggle",
					name = "Manage Fallback",
					get = simpleGet,
					set = simpleSet,
					order = 1,
				},
				fallback_base = {
					type = "select",
					name = "Fallback Base",
					disabled = function() return not profile.manage_fallback end,
					values = FallbackBases,
					get = simpleGet,
					set = simpleSet,
					order = 2,
				},
				fallback_percent = {
					type = "range",
					name = "Fallback %",
					disabled = function() return not profile.manage_threshold end,
					min = 0.0,
					max = 1.0,
					step = 0.01,
					isPercent = true,
					get = simpleGet,
					set = simpleSet,
					order = 6,
				},
			},
		},
	},
}

DB_defaults = {
	profile = {
		manage_threshold = false,
		threshold_percent = 0.15,
		threshold_value = 0,
		threshold_base = 'ItemAuditor',
		auction_house = 0.05,

		manage_fallback = false,
		fallback_base = 'Auctioneer:MP',
		fallback_percent = 0.15
	},
}

function QAManager:OnInitialize()
	self.db = LibStub("AceDB-3.0"):New("QAManagerDB", DB_defaults, true)
	profile = self.db.profile
	self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("QA Manager", "QA Manager")
	LibStub("AceConfig-3.0"):RegisterOptionsTable("QA Manager", Options, {"qam"})

	self:RegisterMessage("QA_NEW_GROUP")
	self:RegisterMessage("QA_GROUP_CHANGED")

	self:RegisterMessage("IA_COST_CHANGED")
	-- self:ShowOptionsGUI()
end

function QAManager:QA_NEW_GROUP(event, groupName)
	local link, itemName = string.match(groupName, "^|c%x+|H(item[-%d:]+)|h%[([^]]+)")

	if itemName then
		link = QAAPI:GetSafeLink(groupName)
		-- groupName = itemName:gsub('Scroll of Enchant', 'SOE:')
		if itemName:match('Scroll of Enchant') then
			groupName = self.renameGroup(groupName, itemName:gsub('Scroll of Enchant', 'SOE:'))
		else
			groupName = self.renameGroup(groupName, itemName)
		end

		QAAPI:AddItem(groupName, link)
	end
end

function QAManager:IA_COST_CHANGED(event, link, total, each, count)
	link = select(2, GetItemInfo(link))
	local groupName = QAAPI:GetItemGroup(link)
	if groupName then
		self:QA_GROUP_CHANGED(event, groupName)
	end
end

function QAManager:QA_GROUP_CHANGED(event, groupName, fromAPI)
	QAManager:UpdateGroupFallback(groupName)
	self:UpdateGroupThreshold(groupName)
end

function QAManager:RefreshQAGroups()
	for groupName in pairs(QAAPI:GetGroups()) do
		self:QA_GROUP_CHANGED(nil, groupName)
	end
end

function QAManager:GetFallbackBase(link)
	local default_value = 99990000
	if profile.fallback_base == 'Auctioneer:MP' then
		local faction = 'Neutral'
		if profile.auction_house == 0.05 then
			faction = UnitFactionGroup("player")
		end
		return AucAdvanced and AucAdvanced.API.GetMarketValue(select(2, GetItemInfo(link)),  GetRealmName()..'-'..faction) or default_value
	elseif profile.fallback_base == 'Auctioneer:Low' then
		return AucAdvanced and select(6, AucAdvanced.Modules.Util.SimpleAuction.Private.GetItems(link)) or default_value
	elseif profile.fallback_base == 'GetAuctionBuyout' then
		return GetAuctionBuyout(link) or default_value
	end
end

-- Highest market value in the group
function QAManager:UpdateGroupFallback(groupName)
	if groupName and profile.manage_fallback then
		local threshold = 0
		for link in pairs(QAAPI:GetItemsInGroup(groupName)) do
			local ah_price = QAManager:GetFallbackBase(link) or 0
			ah_price = ceil(ah_price * (1+profile.fallback_percent))

			threshold = max(threshold, ah_price)
		end

		QAAPI:SetGroupConfig(groupName, 'fallback', threshold)
	end
end

function QAManager:GetThresholdBase(link)
	if profile.threshold_base == 'ItemAuditor' then
		return select(2, IAapi.GetItemCost(link))
	end
	return 0
end

-- Highest threshold in the group
function QAManager:UpdateGroupThreshold(groupName)
	if groupName and profile.manage_threshold then
		local threshold = 0
		for link in pairs(QAAPI:GetItemsInGroup(groupName)) do
			local copper = max(1, QAManager:GetThresholdBase(link))

			local min_by_percent = copper * (1+profile.threshold_percent)
			local min_by_value = copper + profile.threshold_value
			copper = max(min_by_percent, min_by_value)
			copper = copper/(1 - profile.auction_house)

			local vendor = select(11, GetItemInfo(link)) or 0
			threshold = max(threshold, copper, vendor)
		end

		QAAPI:SetGroupConfig(groupName, 'threshold', ceil(threshold))
	end
end

-- @TODO: implement renaming
function QAManager.renameGroup(groupName, newName)
	QAAPI:RenameGroup(groupName, newName)
	return newName
end

function QAManager:ShowOptionsGUI()
	InterfaceOptionsFrame_OpenToCategory(self.optionsFrame)
end

function parseMoney(value)
	local gold = tonumber(string.match(value, "([0-9]+)|c([0-9a-fA-F]+)g|r") or string.match(value, "([0-9]+)g"))
	local silver = tonumber(string.match(value, "([0-9]+)|c([0-9a-fA-F]+)s|r") or string.match(value, "([0-9]+)s"))
	local copper = tonumber(string.match(value, "([0-9]+)|c([0-9a-fA-F]+)c|r") or string.match(value, "([0-9]+)c"))

	if( not gold and not silver and not copper ) then
		return nil;
	end

	-- Convert it all into copper
	return (copper or 0) + ((gold or 0) * COPPER_PER_GOLD) + ((silver or 0) * COPPER_PER_SILVER)
end

function formatMoney(copper, color, textOnly)
	color = color or "|cFFFFFFFF"
	local prefix = ""
	copper = copper or 0
	if copper < 0 then
		prefix = "-"
		copper = abs(copper)
	end

	local copperTexture = COPPER_AMOUNT_TEXTURE
	local silverTexture = SILVER_AMOUNT_TEXTURE
	local goldTexture = GOLD_AMOUNT_TEXTURE
	if textOnly then
		copperTexture = '%dc'
		silverTexture = '%ds'
		goldTexture = '%dg'
	end

	local gold = floor( copper / 10000 );
	copper = mod(copper, 10000)
	local silver = floor( copper / 100 );
	copper = mod(copper, 100)


	copper = color .. format(copperTexture, copper, 13, 13)
	if silver > 0 or gold > 0 then
		silver = color.. format(silverTexture, silver, 13, 13) .. ' '
	else
		silver = ""
	end
	if gold > 0 then
		gold = color.. format(goldTexture, gold, 13, 13) .. ' '
	else
		gold = ""
	end

	return format("%s%s%s%s", prefix, gold, silver, copper)
end